﻿using System;
using REBORN.Common.Plugin.Redis;
using REBORN.CacheManager.DataStatistic;
using REBORN.Common.Plugin;
using REBORN.Common;
using Property.Repository;
using Property.Model.Extend;
using Property.Model.CacheEntity;

namespace Property.CacheManager.User
{
    /// <summary>
    /// redis缓存（只适用于数据统计）
    /// </summary>
    public class UserReidsCache 
    {
        static RedisHelper redis = new RedisHelper(DataConstant.REDIS_DB3);
        /// <summary>
        /// 设置缓存
        /// </summary>
        /// <param name="model"></param>
        public static void UserInfoSet(string cacheKey, UserInfo model, int JwtExpirTime)
        {
            try
            {
                TimeSpan ts = GetExpirTime(JwtExpirTime);
                redis.StringSet<UserInfo>(cacheKey, model, ts);
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "UserInfoSet缓存设置失败");
            }
        }

        /// <summary>
        /// 获取缓存时长
        /// </summary>
        /// <param name="JwtExpirTime"></param>
        /// <returns></returns>
        private static TimeSpan GetExpirTime(int JwtExpirTime)
        {
            DateTime dt = DateTime.Now;
            DateTime dt2 = DateTime.Now;
            TimeSpan ts = dt.AddSeconds(JwtExpirTime) - dt2;
            return ts;
        }

        /// <summary>
        /// 获取缓存
        /// </summary>
        /// <returns></returns>
        public static UserInfo UserInfoGet(string cacheKey)
        {
            UserInfo model = null;
            try
            {
                model = redis.StringGet<UserInfo>(cacheKey);
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "UserInfoSet缓存获取失败");
                return null;
            }            
            return model;
        }

        /// <summary>
        /// 获取用户登录信息
        /// </summary>
        /// <param name="uid"></param>
        /// <returns></returns>
        public static UserInfo GetUserLoginInfo(string uid)
        {

            string cacheKey = CacheKey.UserModuleCacheKeyConfig.USER_Login_Info + uid;
            UserInfo userInfo = null;
            try
            {
                userInfo = redis.StringGet<UserInfo>(cacheKey);
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "GetUserLoginInfo");
            }
            if (userInfo == null)
            {
                RB_AccountRepository repository = new RB_AccountRepository();
                RB_GroupRepository groupRepository = new RB_GroupRepository();
                Int32.TryParse(uid, out int emoloyeeId);
                Employee_ViewModel model = repository.GetEmployeeInfo(emoloyeeId);
                if (model != null)
                {
                    var gmodel = groupRepository.GetEntity(model.Group_Id);
                    string token = "";
                    userInfo = new UserInfo
                    {
                        GroupName = model.GroupName,
                        BranchName = model.SchoolName,
                        DepartName = model.DeptName,
                        PostName = model.PostName,
                        emName = model.EmployeeName,
                        EmployeeId = model.Id,
                        RB_Group_id = model.Group_Id,
                        RB_Branch_id = model.School_Id,
                        RB_Post_Id = model.Post_Id,
                        RB_Department_Id = model.Dept_Id,
                        token = token,
                        Icon = model.UserIcon,
                        Domain = gmodel.Domain
                    };
                    try
                    {
                        TimeSpan ts = GetExpirTime(Config.JwtExpirTime);
                        redis.StringSet<UserInfo>(cacheKey, userInfo, ts);
                    }
                    catch
                    {
                    }
                }
            }
            return userInfo;
        }

        /// <summary>
        /// 根据员工编号获取员工信息
        /// </summary>
        /// <param name="EmployeeId">员工编号</param>
        /// <returns></returns>
        public static CacheUser GetEmployee(object EmployeeId)
        {
            if (EmployeeId != null)
            {
                int.TryParse(EmployeeId.ToString(), out int uid);
                string key = string.Concat(CacheKey.UserCacheKeyConfig.USER_EMPLOYEE, uid.ToString());
                if (uid > 0)
                {
                    if (redis.KeyExists(key))
                    {
                        return redis.StringGet<CacheUser>(key);
                    }
                    else
                    {
                        RB_AccountRepository repository = new RB_AccountRepository();
                        Employee_ViewModel model = repository.GetEmployeeInfo(uid);
                        if (model != null)
                        {
                            var user = new CacheUser()
                            {
                                EmployeeId = model.Id,
                                EmName = model.EmployeeName,
                                RB_Group_id = model.Group_Id,
                                RB_Branch_id = model.School_Id,
                                IsLeave = model.LeaveStatus== REBORN.Common.Enum.LeaveStatusEnum.Departure ? 1 : 0,
                                GroupName = model.GroupName,
                                BranchName = model.SchoolName,
                                DepartName = model.DeptName,
                                PostName = model.PostName,
                                EmPhoto = model.UserIcon,
                                EmMobile = model.Account,
                                LoginMobile = model.Account,
                                RB_Department_Id = model.Dept_Id,
                                RB_Post_Id = model.Post_Id
                            };
                            TimeSpan ts = GetExpirTime((int)REBORN.Common.Enum.CacheTimeEnum.OneDay);
                            redis.StringSet<CacheUser>(key, user, ts);
                            return user;
                        }
                        else
                        {
                            return null;
                        }
                    }
                }
                else
                {
                    return null;
                }
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// 判断key是否存在
        /// </summary>
        /// <param name="cacheKey"></param>
        /// <returns></returns>
        public static bool Exists(string cacheKey) {
            return redis.KeyExists(cacheKey);
        }

        /// <summary>
        /// 设置缓存
        /// </summary>
        /// <param name="cacheKey"></param>
        /// <param name="Data"></param>
        /// <param name="JwtExpirTime"></param>
        public static void Set(string cacheKey, object Data, int JwtExpirTime) {
            try
            {
                TimeSpan ts = GetExpirTime(JwtExpirTime);
                redis.StringSet(cacheKey, Data, ts);
            }
            catch (Exception)
            {
            }
        }
    }
}
