﻿using Dapper;
using Property.Model.Entity;
using Property.Model.Extend;
using REBORN.Common.Enum;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Property.Repository
{
    /// <summary>
    /// 资产入库单仓储类
    /// </summary>
    public partial class RB_Property_StockinRepository : RepositoryBase<RB_Property_Stockin>
    {
        /// <summary>
        /// 获取分页数据
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="dmodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Property_Stockin_Extend> GetPageList(int pageIndex, int pageSize, RB_Property_Stockin_Extend dmodel, out long count)
        {
            string where = " where 1=1 ";
            where += string.Format(" AND ss.{0}={1}", nameof(RB_Property_Stockin_Extend.Status), 0);
            where += $@" and ss.RB_Group_Id={dmodel.RB_Group_Id} ";            
            if (!string.IsNullOrWhiteSpace(dmodel.StartTime))
            {
                where += $@" and ss.{nameof(RB_Property_Stockin_Extend.StockInDate)} >='{dmodel.StartTime}'";
            }
            if (!string.IsNullOrWhiteSpace(dmodel.EndTime))
            {
                where += $@" and ss.{nameof(RB_Property_Stockin_Extend.StockInDate)} <='{dmodel.EndTime + " 23:59:59"}'";
            }
            if (!string.IsNullOrWhiteSpace(dmodel.StockInNum))
            {
                where += " and ss." + nameof(RB_Property_Stockin_Extend.StockInNum) + " like '%" + dmodel.StockInNum + "%'";
            }
            if (!string.IsNullOrEmpty(dmodel.SupplierName))
            {
                where += " and ss." + nameof(RB_Property_Stockin_Extend.SupplierName) + " like '%" + dmodel.SupplierName + "%'";
            }
            if (!string.IsNullOrEmpty(dmodel.PropertyName))
            {
                where += " and sd." + nameof(RB_Property_StockinDetail_Extend.PropertyName) + " like '%" + dmodel.PropertyName + "%'";
            }
            if (dmodel.ProcurementId > 0)
            {
                where += " and ss."+ nameof(RB_Property_Stockin_Extend.ProcurementId) + " =" + dmodel.ProcurementId;
            }
            if (dmodel.StockInState > 0)
            {
                where += " and ss." + nameof(RB_Property_Stockin_Extend.StockInState) + "=" + (int)dmodel.StockInState;
            }
            
            string sql = $@" select ss.*,pp.FinanceId from RB_Property_Stockin ss
left join rb_property_stockindetail sd on ss.Id = sd.StockInId
left join rb_property_procurement pp on pp.Id = ss.ProcurementId
{where} group by ss.Id order by ss.Id desc";
            return GetPage<RB_Property_Stockin_Extend>(pageIndex, pageSize, out count, sql).ToList();
        }

        /// <summary>
        /// 获取耗材分页数据
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Property_Stockin_Extend> GetList(RB_Property_Stockin_Extend dmodel)
        {
            string where = " where 1=1 ";
            where += string.Format(" AND ss.{0}={1}", nameof(RB_Property_Stockin_Extend.Status), 0);
            where += $@" and ss.RB_Group_Id={dmodel.RB_Group_Id} ";
            if (!string.IsNullOrWhiteSpace(dmodel.StartTime))
            {
                where += $@" and ss.{nameof(RB_Property_Stockin_Extend.StockInDate)} >='{dmodel.StartTime}'";
            }
            if (!string.IsNullOrWhiteSpace(dmodel.EndTime))
            {
                where += $@" and ss.{nameof(RB_Property_Stockin_Extend.StockInDate)} <='{dmodel.EndTime + " 23:59:59"}'";
            }
            if (!string.IsNullOrWhiteSpace(dmodel.StockInNum))
            {
                where += " and ss." + nameof(RB_Property_Stockin_Extend.StockInNum) + " like '%" + dmodel.StockInNum + "%'";
            }
            if (!string.IsNullOrEmpty(dmodel.SupplierName))
            {
                where += " and ss." + nameof(RB_Property_Stockin_Extend.SupplierName) + " like '%" + dmodel.SupplierName + "%'";
            }
            if (dmodel.ProcurementId > 0)
            {
                where += " and ss." + nameof(RB_Property_Stockin_Extend.ProcurementId) + " =" + dmodel.ProcurementId;
            }
            if (dmodel.StockInState > 0)
            {
                where += " and ss." + nameof(RB_Property_Stockin_Extend.StockInState) + "=" + (int)dmodel.StockInState;
            }

            string sql = $@" select ss.* from RB_Property_Stockin ss
{where} group by ss.Id order by ss.Id desc";
            return Get<RB_Property_Stockin_Extend>(sql).ToList();
        }


        /// <summary>
        /// 获取待我审批分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="dmodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Property_Stockin_Extend> GetStockInWaitMyAuditPageList(int pageIndex, int pageSize, RB_Property_Stockin_Extend dmodel, out long count)
        {
            var parameters = new DynamicParameters();
            string where = " ";
            where += string.Format(" AND s.{0}={1}", nameof(RB_Property_Stockin_Extend.Status), 0);
            where += $@" and s.RB_Group_Id={dmodel.RB_Group_Id}";

            if (!string.IsNullOrEmpty(dmodel.StockInNum))
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.StockInNum) + " like @StockInNum";
                parameters.Add("StockInNum", "%" + dmodel.StockInNum + "%");
            }
            if (!string.IsNullOrEmpty(dmodel.StartTime))
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.CreateDate) + " >='" + dmodel.StartTime + "'";
            }
            if (!string.IsNullOrEmpty(dmodel.EndTime))
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.CreateDate) + " <='" + dmodel.EndTime + " 23:59:59'";
            }
            if (dmodel.StockInState > 0)
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.StockInState) + "=" + (int)dmodel.StockInState;
            }
            if (dmodel.CreateBy > 0)
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.CreateBy) + "=" + dmodel.CreateBy;
            }
            string whereTHC = "";
            string whereTZC = "";
            if (dmodel.AuditEmId > 0)
            {
                where += " and wa." + nameof(Rb_Workflow_Auditrecord.AuditEmId) + "=" + dmodel.AuditEmId;
                where += " and wa." + nameof(Rb_Workflow_Auditrecord.AuditStatus) + "=" + (int)WFRrocessStatus.NotAudit;
                where += " and s." + nameof(RB_Property_Stockin_Extend.StockInState) + "=" + (int)StockInStatusEnum.StockInWarting;//需要申请单在审核中
                where += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.SpecialNode) + " !=" + (int)WFSpecialNodeEnum.Pay;//排除签名的
                whereTHC += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.TemplateType) + " =3";
                whereTZC += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.TemplateType) + " =4";
            }
            if (dmodel.AuditedEmId > 0)
            {
                where += " and wa." + nameof(Rb_Workflow_Auditrecord.AuditEmId) + "=" + dmodel.AuditedEmId;
                where += " and wa." + nameof(Rb_Workflow_Auditrecord.AuditStatus) + " in(" + (int)WFRrocessStatus.AuditThrough + "," + (int)WFRrocessStatus.AuditNotThrough + "," + (int)WFRrocessStatus.Rejected + ")";
                where += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.SpecialNode) + " !=" + (int)WFSpecialNodeEnum.Pay;//排除签名的
                whereTHC += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.TemplateType) + " =3";
                whereTZC += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.TemplateType) + " =4";
            }

            string where2 = "";
            if (dmodel.StockInType > 0)
            {
                where2 = $@" and t.StockInType ={dmodel.StockInType}";
            }

            string sql = $@" 
SELECT * FROM(
SELECT s.Id, 1 as StockInType,s.StockInDate,s.StockInNum,s.StockInState,s.CreateBy,s.UpdateDate FROM rb_property_stockin s
left join rb_workflow_auditrecord wa on s.Id=wa.WorkFlowId
LEFT JOIN rb_workflow_auditrelevance war on wa.AuditId=war.Id
WHERE 1=1 {where} {whereTZC}
GROUP BY s.Id

UNION

SELECT s.Id, 2 as StockInType,s.StockInDate,s.StockInNum,s.StockInState,s.CreateBy,s.UpdateDate FROM rb_supplies_stockin s
left join rb_workflow_auditrecord wa on s.Id=wa.WorkFlowId
LEFT JOIN rb_workflow_auditrelevance war on wa.AuditId=war.Id
WHERE 1=1 {where} {whereTHC}
GROUP BY s.Id
)t WHERE 1=1 {where2} ORDER BY t.UpdateDate DESC
";
            return GetPage<RB_Property_Stockin_Extend>(pageIndex, pageSize, out count, sql).ToList();
        }

        /// <summary>
        /// 获取待我审批数量
        /// </summary>
        /// <param name="dmodel"></param>        
        /// <returns></returns>
        public int GetStockInWaitMyAuditCount(RB_Property_Stockin_Extend dmodel)
        {
            var parameters = new DynamicParameters();
            string where = " ";
            where += string.Format(" AND s.{0}={1}", nameof(RB_Property_Stockin_Extend.Status), 0);
            where += $@" and s.RB_Group_Id={dmodel.RB_Group_Id}";

            if (!string.IsNullOrEmpty(dmodel.StockInNum))
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.StockInNum) + " like @StockInNum";
                parameters.Add("StockInNum", "%" + dmodel.StockInNum + "%");
            }
            if (!string.IsNullOrEmpty(dmodel.StartTime))
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.CreateDate) + " >='" + dmodel.StartTime + "'";
            }
            if (!string.IsNullOrEmpty(dmodel.EndTime))
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.CreateDate) + " <='" + dmodel.EndTime + " 23:59:59'";
            }
            if (dmodel.StockInState > 0)
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.StockInState) + "=" + (int)dmodel.StockInState;
            }
            if (dmodel.CreateBy > 0)
            {
                where += " and s." + nameof(RB_Property_Stockin_Extend.CreateBy) + "=" + dmodel.CreateBy;
            }
            string whereTHC = "";
            string whereTZC = "";
            if (dmodel.AuditEmId > 0)
            {
                where += " and wa." + nameof(Rb_Workflow_Auditrecord.AuditEmId) + "=" + dmodel.AuditEmId;
                where += " and wa." + nameof(Rb_Workflow_Auditrecord.AuditStatus) + "=" + (int)WFRrocessStatus.NotAudit;
                where += " and s." + nameof(RB_Property_Stockin_Extend.StockInState) + "=" + (int)StockInStatusEnum.StockInWarting;//需要申请单在审核中
                where += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.SpecialNode) + " !=" + (int)WFSpecialNodeEnum.Pay;//排除签名的
                whereTHC += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.TemplateType) + " =3";
                whereTZC += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.TemplateType) + " =4";
            }
            if (dmodel.AuditedEmId > 0)
            {
                where += " and wa." + nameof(Rb_Workflow_Auditrecord.AuditEmId) + "=" + dmodel.AuditedEmId;
                where += " and wa." + nameof(Rb_Workflow_Auditrecord.AuditStatus) + " in(" + (int)WFRrocessStatus.AuditThrough + "," + (int)WFRrocessStatus.AuditNotThrough + "," + (int)WFRrocessStatus.Rejected + ")";
                where += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.SpecialNode) + " !=" + (int)WFSpecialNodeEnum.Pay;//排除签名的
                whereTHC += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.TemplateType) + " =3";
                whereTZC += " and war." + nameof(Rb_Workflow_AuditRelevance_Extend.TemplateType) + " =4";
            }

            string sql = $@"
SELECT count(*) as Number FROM(
SELECT s.Id, 1 as Type,s.StockInDate,s.StockInNum,s.StockInState FROM rb_property_stockin s
left join rb_workflow_auditrecord wa on s.Id=wa.WorkFlowId
LEFT JOIN rb_workflow_auditrelevance war on wa.AuditId=war.Id
WHERE 1=1 {where} {whereTZC}
GROUP BY s.Id

UNION

SELECT s.Id, 2 as Type,s.StockInDate,s.StockInNum,s.StockInState FROM rb_supplies_stockin s
left join rb_workflow_auditrecord wa on s.Id=wa.WorkFlowId
LEFT JOIN rb_workflow_auditrelevance war on wa.AuditId=war.Id
WHERE 1=1 {where} {whereTHC}
GROUP BY s.Id
)t ";
            var obj = ExecuteScalar(sql);
            return obj == null ? 0 : Convert.ToInt32(obj);
        }

    }
}
