﻿using System.Linq;
using Microsoft.AspNetCore.Mvc;
using REBORN.Common.API;
using Newtonsoft.Json;
using Property.Module.UserModule;
using Property.Model.Extend;
using Property.CacheKey;
using REBORN.Common;
using Property.WebApi.Filter;
using Newtonsoft.Json.Linq;
using REBORN.Common.Plugin;
using Property.Model.Entity;
using System;
using System.Collections.Generic;
using JWT;
using JWT.Algorithms;
using JWT.Serializers;
using Microsoft.AspNetCore.Cors;
using REBORN.Common.Enum;

namespace Property.WebApi.Controllers.Login
{
    [Route("api/[controller]/[action]")]    
    [ApiExceptionFilter]    
    [ApiController]
    [EnableCors("AllowCors")]
    public class LoginController : ControllerBase
    {
        /// <summary>
        /// 账号管理处理类
        /// </summary>
        private readonly AccountModule accountModule = new AccountModule();
        /// <summary>
        /// 菜单处理类对象
        /// </summary>
        private readonly MenuModule menuModule = new MenuModule();
        /// <summary>
        /// 集团处理类
        /// </summary>
        private readonly GroupModule groupModule = new GroupModule();



        /// <summary>
        /// 用户登录
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult UserLogin(object requestMsg)
        {
            var requestParm = JsonConvert.DeserializeObject<RequestParm>(requestMsg.ToString());
            JObject jobj = JObject.Parse(requestParm.msg.ToString());
            string account = jobj.GetStringValue("EmAccount");
            string password = jobj.GetStringValue("EmPassword");
            string Domain = jobj.GetStringValue("Domain");
            int accountType = jobj.GetInt("AccountType", 1);

            if (Domain.Contains("zc"))
            {
                Domain = Domain.Replace("zc", "");//资产管理专用
            }
            var groupId = groupModule.GetGroupIdByDomainModule(Domain);
            if (groupId == 0)
            {
                return ApiResult.Failed("集团域名不存在，登录失败");
            }

            var model = accountModule.GetAccountListExtModule(new RB_Account_ViewModel()
            {
                Account = account.Trim(),
                Group_Id = groupId
            })?.FirstOrDefault();
            if (model == null)
            {
                return ApiResult.Failed(message: $"未找到【{ account }】用户!");
            }
            else
            {
                if (password != "Viitto!@#123")
                {
                    password = DES.Encrypt(password);
                    if (model.Password != password)
                    {
                        return ApiResult.Failed("密码错误");
                    }
                }
                if (model.Status == DateStateEnum.Delete)
                {
                    return ApiResult.Failed(message: $"此账号【{ account }】已禁用,如需使用请联系管理员!");
                }
                if (model.LeaveStatus == 4)
                {
                    return ApiResult.Failed(message: $"此账号【{ account }】已离职,无法登陆!");
                }

                var treeList = menuModule.GetPostMenuTreeModule(new RB_Menu_ViewModel()
                {
                    MenuType = accountType
                }, out List<RB_Menu_ViewModel> userCenterList, postIds: model.Post_Id.ToString());


                TokenUserInfo userInfo = new TokenUserInfo { uid = model.Id.ToString(), requestFrom = ApiRequestFromEnum.ERP };
                string token = "";
                #region JWT
                IDateTimeProvider provider = new UtcDateTimeProvider();
                var now = provider.GetNow().AddMinutes(-1);
                var unixEpoch = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc); // or use JwtValidator.UnixEpoch
                var secondsSinceEpoch = Math.Round((now - unixEpoch).TotalSeconds);
                var payload = new Dictionary<string, object>
                                {
                                        {"iat",secondsSinceEpoch },
                                        {"exp",secondsSinceEpoch+Config.JwtExpirTime},
                                        {"edu_userInfo",userInfo }
                                };
                IJwtAlgorithm algorithm = new HMACSHA256Algorithm();
                IJsonSerializer serializer = new JsonNetSerializer();
                IBase64UrlEncoder urlEncoder = new JwtBase64UrlEncoder();
                IJwtEncoder encoder = new JwtEncoder(algorithm, serializer, urlEncoder);
                string secret = Config.JwtSecretKey;
                token = encoder.Encode(payload, secret);
                #endregion

                UserInfo obj = new UserInfo
                {

                    GroupName = model.GroupName,
                    GroupPic = model.GroupLogo,
                    Domain = "zc" + Domain,
                    BranchName = model.SchoolName,
                    DepartName = model.DeptName,
                    PostName = model.PostName,
                    emName = model.AccountName,
                    EmployeeId = model.Id,
                    RB_Group_id = model.Group_Id,
                    RB_Branch_id = model.School_Id,
                    RB_Post_Id = model.Post_Id,
                    RB_Department_Id = model.Dept_Id,
                    token = token,                    
                    FileDomain = Config.GetOssFileUrl,
                    Icon = model.UserIcon
                };
                CacheManager.User.UserReidsCache.UserInfoSet(UserModuleCacheKeyConfig.USER_Login_Info + model.Id, obj, Config.JwtExpirTime);
                obj.UserMenu = treeList;
                return ApiResult.Success(data: obj);
            }
        }

        /// <summary>
        /// 企业微信自动登录
        /// </summary>
        /// <param name="requestMsg"></param>
        /// <returns></returns>
        [HttpPost]
        public ApiResult UserLoginQY(object requestMsg) {
            var requestParm = JsonConvert.DeserializeObject<RequestParm>(requestMsg.ToString());
            JObject jobj = JObject.Parse(requestParm.msg.ToString());
            int loginId = jobj.GetInt("LoginId", 0);

            var model = accountModule.GetAccountListExtModule(new RB_Account_ViewModel()
            {
                Id = loginId
            })?.FirstOrDefault();
            if (model == null)
            {
                return ApiResult.Failed(message: $"未找到【{ loginId }】用户!");
            }
            else
            {
                if (model.Status == DateStateEnum.Delete)
                {
                    return ApiResult.Failed(message: $"此账号【{ loginId }】已禁用,如需使用请联系管理员!");
                }
                if (model.LeaveStatus == 4)
                {
                    return ApiResult.Failed(message: $"此账号【{ loginId }】已离职,无法登陆!");
                }

                var treeList = menuModule.GetPostMenuTreeModule(new RB_Menu_ViewModel()
                {
                    MenuType = 1
                }, out List<RB_Menu_ViewModel> userCenterList, postIds: model.Post_Id.ToString());


                TokenUserInfo userInfo = new TokenUserInfo { uid = model.Id.ToString(), requestFrom = ApiRequestFromEnum.ERP };
                string token = "";
                #region JWT
                IDateTimeProvider provider = new UtcDateTimeProvider();
                var now = provider.GetNow().AddMinutes(-1);
                var unixEpoch = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc); // or use JwtValidator.UnixEpoch
                var secondsSinceEpoch = Math.Round((now - unixEpoch).TotalSeconds);
                var payload = new Dictionary<string, object>
                                {
                                        {"iat",secondsSinceEpoch },
                                        {"exp",secondsSinceEpoch+Config.JwtExpirTime},
                                        {"edu_userInfo",userInfo }
                                };
                IJwtAlgorithm algorithm = new HMACSHA256Algorithm();
                IJsonSerializer serializer = new JsonNetSerializer();
                IBase64UrlEncoder urlEncoder = new JwtBase64UrlEncoder();
                IJwtEncoder encoder = new JwtEncoder(algorithm, serializer, urlEncoder);
                string secret = Config.JwtSecretKey;
                token = encoder.Encode(payload, secret);
                #endregion

                UserInfo obj = new UserInfo
                {

                    GroupName = model.GroupName,
                    GroupPic = model.GroupLogo,
                    Domain = "zc",
                    BranchName = model.SchoolName,
                    DepartName = model.DeptName,
                    PostName = model.PostName,
                    emName = model.AccountName,
                    EmployeeId = model.Id,
                    RB_Group_id = model.Group_Id,
                    RB_Branch_id = model.School_Id,
                    RB_Post_Id = model.Post_Id,
                    RB_Department_Id = model.Dept_Id,
                    token = token,
                    FileDomain = Config.GetOssFileUrl,
                    Icon = model.UserIcon
                };
                CacheManager.User.UserReidsCache.UserInfoSet(UserModuleCacheKeyConfig.USER_Login_Info + model.Id, obj, Config.JwtExpirTime);
                obj.UserMenu = treeList;
                return ApiResult.Success(data: obj);
            }
        }
    }
}
