﻿using System.Linq;
using Microsoft.AspNetCore.Mvc;
using REBORN.Common.API;
using Newtonsoft.Json;
using Property.Model.Extend;
using REBORN.Common;
using Property.Module.UserModule;
using Newtonsoft.Json.Linq;
using REBORN.Common.Plugin;
using System.Collections.Generic;

namespace Property.WebApi.Controllers.User
{
    public class UserController : PApiController
    {
        //员工
        private readonly EmployeeModule employeeModule = new EmployeeModule();
        //公司
        private readonly SchoolModule schoolModule = new SchoolModule();
        //部门
        private readonly DepartmentModule departmentModule = new DepartmentModule();
        /// <summary>
        /// 菜单处理类对象
        /// </summary>
        private readonly MenuModule menuModule = new MenuModule();


        /// <summary>
        /// 获取公司列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetBranchList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            var list = schoolModule.GetSchoolListModule(new RB_School_ViewModel()
            {
                Group_Id = userInfo.RB_Group_id
            });
            return ApiResult.Success("", list.Select(x => new
            {
                Id = x.SId,
                BName = x.SName
            }));
        }

        /// <summary>
        /// 获取员工列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetEmployeeList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            Employee_ViewModel demodel = JsonConvert.DeserializeObject<Employee_ViewModel>(requestParm.msg.ToString());

            demodel.Group_Id = userInfo.RB_Group_id;
            demodel.EmployeeName = demodel.EmName;
            var list = employeeModule.GetEmployeeListModule(demodel);
            var robj = list.Select(x => new
            {
                EmployeeId = x.Id,
                EmName = x.EmployeeName,
                x.AccountType
            });
            return ApiResult.Success("", robj);
        }

        /// <summary>
        /// 获取部门树形结构
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetDepartmentTreeForReceiveQuery()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int School_Id = parms.GetInt("School_Id", 0);
            var query = new RB_Department_ViewModel()
            {
                Group_Id = userInfo.RB_Group_id,
                School_Id = School_Id
            };
            var list = departmentModule.GetDepartmentTreeModule(query);
            return ApiResult.Success("", list);
        }

        /// <summary>
        /// 获取用户登录菜单（教育直接跳转资产管理）
        /// </summary>
        /// <param name="requestMsg"></param>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetUserLoginMenu()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int accountType = parms.GetInt("AccountType", 1);

            var treeList = menuModule.GetPostMenuTreeModule(new RB_Menu_ViewModel()
            {
                MenuType = accountType
            }, out List<RB_Menu_ViewModel> userCenterList, postIds: userInfo.RB_Post_Id.ToString());

            userInfo.UserMenu = treeList;
            userInfo.token = requestParm.token;
            return ApiResult.Success(data: userInfo);
        }

    }
}
